<?php

declare(strict_types=1);

namespace Drupal\sm\Transport\DrupalSql;

use Drupal\Core\Database\Connection as CoreConnection;
use Drupal\Core\State\StateInterface;

/**
 * Track message tables created by SM.
 *
 * @internal
 */
final class MessageTableTracker {

  private const SM_TABLES = 'sm_tables';

  public function __construct(
    private readonly StateInterface $state,
  ) {
  }

  /**
   * Tracks a message table created for messages.
   */
  public function addMessageTable(string $table_name): void {
    /** @var array<string> $tables */
    $tables = $this->state->get(self::SM_TABLES, []);
    $tables[$table_name] = $table_name;
    $this->state->set(self::SM_TABLES, $tables);
  }

  /**
   * Drop all database tables created by SM.
   *
   * Must be static to be called from hook_uninstall().
   *
   * @see \sm_uninstall()
   */
  public static function dropAll(): void {
    /** @var array<string> $tables */
    $tables = static::state()->get(self::SM_TABLES, []);

    foreach ($tables as $table) {
      if (static::connection()->schema()->tableExists($table)) {
        static::connection()->schema()->dropTable($table);
      }
    }

    // Clean up the state.
    static::state()->delete(self::SM_TABLES);
  }

  private static function state(): StateInterface {
    return \Drupal::state();
  }

  private static function connection(): CoreConnection {
    return \Drupal::database();
  }

}
