<?php

declare(strict_types=1);

namespace Drupal\sm\Transport\DrupalSql;

/**
 * @internal
 */
final class Schema {

  /**
   * Defines the schema for the messages table.
   *
   * @phpstan-return array{'description': string, 'fields': array<mixed>, 'primary key': string[], 'indexes': array<string, string[]>}
   */
  public static function schemaDefinition(): array {
    return [
      // [DrupalSM] is applied to signify ownership by SM.
      'description' => 'Store Messenger messages. [DrupalSM]',
      'fields' => [
        'id' => [
          'type' => 'serial',
          'size' => 'big',
          'not null' => TRUE,
        ],
        'body' => [
          'type' => 'text',
          'size' => 'big',
          'not null' => TRUE,
        ],
        'headers' => [
          'type' => 'text',
          'size' => 'big',
          'not null' => TRUE,
        ],
        'queue_name' => [
          'type' => 'varchar',
          'length' => 190,
          'not null' => TRUE,
        ],
        'created_at' => [
          'type' => 'varchar',
          'length' => 20,
        ],
        'available_at' => [
          'type' => 'varchar',
          'length' => 20,
        ],
        'delivered_at' => [
          'type' => 'varchar',
          'length' => 20,
        ],
      ],
      'primary key' => ['id'],
      'indexes' => [
        'delivered_at' => ['delivered_at'],
        'available_at' => ['available_at'],
        'queue_name' => ['queue_name'],
      ],
    ];
  }

}
