<?php

declare(strict_types=1);

namespace Drupal\sm\Transport\DrupalSql;

use Symfony\Component\Messenger\Envelope;
use Symfony\Component\Messenger\Exception\TransportException;
use Symfony\Component\Messenger\Stamp\DelayStamp;
use Symfony\Component\Messenger\Stamp\TransportMessageIdStamp;
use Symfony\Component\Messenger\Transport\Sender\SenderInterface;
use Symfony\Component\Messenger\Transport\Serialization\SerializerInterface;

/**
 * A sender for the Drupal SQL transport.
 *
 * @internal
 */
final class Sender implements SenderInterface {

  /**
   * @param \Drupal\sm\Transport\DrupalSql\Connection $connection
   *   The database connection.
   * @param \Symfony\Component\Messenger\Transport\Serialization\SerializerInterface $serializer
   *   A serializer to encode the envelope when it is serialized to the table.
   */
  public function __construct(
    private readonly Connection $connection,
    private readonly SerializerInterface $serializer,
  ) {
  }

  public function send(Envelope $envelope): Envelope {
    $encoded = $this->serializer->encode($envelope);

    $delayStamp = $envelope->last(DelayStamp::class);
    $delay = $delayStamp?->getDelay() ?? 0;

    try {
      $id = $this->connection->send($encoded['body'], $encoded['headers'] ?? [], $delay);
    }
    catch (\Exception $exception) {
      throw new TransportException($exception->getMessage(), previous: $exception);
    }

    return $envelope->with(new TransportMessageIdStamp($id));
  }

}
