<?php

declare(strict_types=1);

namespace Drupal\sm\Transport\DrupalSql;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection as CoreConnection;
use Symfony\Component\Messenger\Transport\Serialization\SerializerInterface;
use Symfony\Component\Messenger\Transport\TransportFactoryInterface;
use Symfony\Component\Messenger\Transport\TransportInterface;

/**
 * Messenger transport factory for Drupal SQL.
 *
 * @implements TransportFactoryInterface<\Drupal\sm\Transport\DrupalSql\Transport>
 * @phpstan-import-type DrupalSqlConfiguration from \Drupal\sm\Transport\DrupalSql\Connection
 * @phpstan-type DrupalSqlConfigurationFromPass array{table_name: string, queue_name: string, redeliver_timeout: positive-int, auto_setup: bool, transport_name: string}
 * @internal
 */
final class TransportFactory implements TransportFactoryInterface {

  public function __construct(
    private readonly CoreConnection $connection,
    private readonly TimeInterface $time,
    private readonly MessageTableTracker $messageTableTracker,
  ) {
  }

  /**
   * @phpstan-param DrupalSqlConfigurationFromPass $options
   * @phpstan-ignore-next-line
   */
  public function createTransport(
    string $dsn,
    array $options,
    SerializerInterface $serializer,
  ): TransportInterface {
    // Clean up the transport_name added by SmCompilerPass.
    unset($options['transport_name']);
    return new Transport(
      $this->connection,
      $serializer,
      Connection::buildConfiguration($dsn, $options),
      $this->time,
      $this->messageTableTracker,
    );
  }

  /**
   * @phpstan-ignore-next-line
   */
  public function supports(string $dsn, array $options): bool {
    /** @var DrupalSqlConfiguration $options */
    return \str_starts_with($dsn, 'drupal-sql://');
  }

}
