<?php

declare(strict_types=1);

namespace Drupal\sm_test;

use Drupal\Component\Datetime\TimeInterface;

/**
 * Service used to simulate time.
 */
final class TimeMachine implements TimeInterface {

  private \DateTimeInterface $time;

  private int $start;

  public function __construct(string $time) {
    $this->time = new \DateTimeImmutable($time);
    $this->start = $this->hrtime();
  }

  /**
   * {@inheritdoc}
   */
  public function getRequestTime() {
    return $this->time->getTimestamp();
  }

  /**
   * {@inheritdoc}
   */
  public function getRequestMicroTime() {
    return (float) $this->time->getTimestamp();
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentTime() {
    return $this->time->getTimestamp() + $this->offsetSinceStart();
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentMicroTime() {
    return (float) $this->time->getTimestamp() + $this->offsetSinceStart();
  }

  private function hrtime(): int {
    return (int) (\hrtime(TRUE) / 1e9);
  }

  private function offsetSinceStart(): int {
    return $this->hrtime() - $this->start;
  }

}
