<?php

declare(strict_types=1);

namespace Drupal\Tests\sm\Kernel;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\KernelTests\KernelTestBase;
use Drupal\sm_test\SmTestMessage;
use Drupal\sm_test\SmTestServiceMessage;
use Drupal\Tests\sm\Traits\SmMappingTrait;
use Symfony\Component\Messenger\MessageBusInterface;

/**
 * Tests the `synchronous` (sync://) transport in sm.services.yml.
 */
final class SmSynchronousTransportTest extends KernelTestBase {

  use SmMappingTrait;

  protected static $modules = [
    'sm',
    'sm_test',
    'sm_test_transport',
  ];

  /**
   * Tests the message is handled synchronously (immediately)
   */
  public function testSynchronousTransport(): void {
    $this->setRouting([
      SmTestMessage::class => 'sm.transport.synchronous',
      // Everything else will be asynchronous.
      // This asynchronous transport is set otherwise messenger routes unmatched
      // message as synchronous.
      '*' => 'sm_test_transport__in_memory',
    ]);

    $synchronousMessage = new SmTestMessage();
    $this->bus()->dispatch($synchronousMessage);
    $asynchronousMessage = new SmTestServiceMessage();
    $this->bus()->dispatch($asynchronousMessage);

    // Handled since its async.
    static::assertNotNull($synchronousMessage->handledBy);
    // This one isn't handled yet.
    static::assertNull($asynchronousMessage->handledBy);
  }

  /**
   * Default messenger bus.
   */
  private function bus(): MessageBusInterface {
    /** @var \Symfony\Component\Messenger\MessageBusInterface */
    return \Drupal::service(MessageBusInterface::class);
  }

  public function register(ContainerBuilder $container): void {
    parent::register($container);
    static::setSmRoutingToContainer($container);
  }

}
