<?php

declare(strict_types=1);

namespace Drupal\Tests\sm\Traits;

use Drupal\Core\DependencyInjection\ContainerBuilder;

/**
 * Trait for managing routing via container parameters.
 *
 * @phpstan-require-implements \Drupal\Core\DependencyInjection\ServiceProviderInterface
 * @phpstan-type RoutingArray array<string|class-string, string|string[]>
 */
trait SmMappingTrait {

  /**
   * @var RoutingArray|null
   */
  private ?array $messageToSendersMapping = NULL;

  protected function setSmRoutingToContainer(ContainerBuilder $container): void {
    if (NULL !== $this->messageToSendersMapping) {
      $container->setParameter('sm.routing', $this->messageToSendersMapping);
    }
  }

  /**
   * Sets routing and triggers a rebuild.
   *
   * @phpstan-param RoutingArray $messageToSendersMapping
   */
  protected function setRouting(array $messageToSendersMapping): void {
    $this->messageToSendersMapping = $messageToSendersMapping;
    \Drupal::service('kernel')->rebuildContainer();
  }

}
