<?php

declare(strict_types=1);

namespace Drupal\Tests\sm\Unit;

use Drupal\sm\Transport\DrupalSql\Connection;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\sm\Transport\DrupalSql\Connection
 * @phpstan-import-type DrupalSqlConfiguration from \Drupal\sm\Transport\DrupalSql\Connection
 */
final class DrupalSqlConnectionUnitTest extends UnitTestCase {

  /**
   * @phpstan-param class-string<\Throwable> $assertException
   * @phpstan-param DrupalSqlConfiguration|array<string, mixed> $options
   * @phpstan-param DrupalSqlConfiguration|array<string, mixed> $expectedConfiguration
   * @dataProvider providerBuildConfiguration
   */
  public function testBuildConfiguration(string $dsn, array $options, ?string $assertException = NULL, ?string $assertExceptionMessage = NULL, array $expectedConfiguration = []): void {
    if ($assertException !== NULL && $assertExceptionMessage !== NULL) {
      static::expectException($assertException);
      static::expectExceptionMessage($assertExceptionMessage);
    }
    $configuration = Connection::buildConfiguration($dsn, $options);
    static::assertEquals($expectedConfiguration, $configuration);
  }

  public static function providerBuildConfiguration(): \Generator {
    yield ['drupal-sql://default?auto_setup=false', [], NULL, '', [
      'table_name' => 'sm_messages',
      'queue_name' => 'default',
      'redeliver_timeout' => 3600,
      'auto_setup' => FALSE,
    ],
    ];
    yield ['drupal-sql://default', [], NULL, '', [
      'table_name' => 'sm_messages',
      'queue_name' => 'default',
      'redeliver_timeout' => 3600,
      'auto_setup' => TRUE,
    ],
    ];
    yield ['drupal-sql://default', [
      'table_name' => 'foo',
      'queue_name' => 'bar',
      'redeliver_timeout' => 60,
      'auto_setup' => TRUE,
    ], NULL, '', [
      'table_name' => 'foo',
      'queue_name' => 'bar',
      'redeliver_timeout' => 60,
      'auto_setup' => TRUE,
    ],
    ];
    yield ['drupal-sql://other-connection', [], \InvalidArgumentException::class, 'Only the `default` connection is supported at this time.'];
    yield ['foobar://default', [], \InvalidArgumentException::class, 'DSN must start with "drupal-sql://"'];
    yield ['drupal-sql://default', ['zim' => 'zam'], \InvalidArgumentException::class, 'Unknown option found: [zim]. Allowed options are [table_name, queue_name, redeliver_timeout, auto_setup]'];
    yield ['drupal-sql://default?zim=zam', [], \InvalidArgumentException::class, 'Unknown option found in DSN: [zim]. Allowed options are [table_name, queue_name, redeliver_timeout, auto_setup].'];
  }

}
